# Copyright (c) 2001, Gemini Data Loggers (UK) Ltd

import os, new, sys, traceback

import ZODB.FileStorage

import ReplicatedFile

# FileStorage doesnt provide the hooks we need to use a different type of file.
#
# For a less invasive way of adding this feature, see Shane Hathaway's
# http://www.zope.org/Members/hathawsh/PartitionedFileStorage 
# 
# For now, I am content with forcing attributes into ZODB.FileStorage
# module namespace. Its main advantage is that I can use an unmodified
# FileStorage.py

replicated_files = []

def register_filestorage_filename(f):
    replicated_files.append(f)
    replicated_files.append(f+'.pack')


def fix():
    global fsync
    try: from posix import fsync
    except: fsync=None

    assert ZODB.FileStorage.os is os,                       'Someone else stole his attributes first'
    assert ZODB.FileStorage.__builtins__['open'] is open,   'Someone else stole his attributes first'
    assert ZODB.FileStorage.fsync is fsync,                 'Someone else stole his attributes first'

    new_os = new.module('os(faked by ReplicatedFile ZEO module)')

    for name in dir(os):
        setattr(new_os,name,getattr(os,name))

    new_os.unlink = new_unlink
    new_os.remove = new_unlink
    new_os.rename = new_rename


    ZODB.FileStorage.os = new_os
    ZODB.FileStorage.open = new_open
    ZODB.FileStorage.fsync = new_fsync

def new_unlink(*args):
    debug('unlink',args)
    return apply(os.unlink,args)

def new_open(*args):
    debug('open',args)
    if args[0] in replicated_files:
        if '+' in args[1] or 'w' in args[1]:
            return apply(ReplicatedFile.File,args)
    return apply(open,args)

def new_fsync(*args):
    debug('sync',args)
    if hasattr(args[0],'fsync'):
        args[0].fsync()
    else:
        if fsync:
            return apply(fsync,args)

def new_rename(*args):
    debug('rename',args)
    if args[0] in replicated_files:
        try:
            f = ReplicatedFile.file_map[os.path.split(args[0])[1]]
            close_it = 0
        except KeyError:
            f = ReplicatedFile.File(args[0],'r+b')
            close_it = 1
        f.rename(args[1])
        if close_it:
            f.close()
    else:
        return apply(os.rename,args)


def debug(fn,args):
    if 0:
        print >> sys.stderr, fn, args
